/*
 * Generic access to memory card functionality.
 *
 * Copyright (C) 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <string.h>
#include "msc-generic.h"

/* Function declarations reproduced here to avoid clock identifier conflicts in
   the chip-specific headers. */

//extern Msc_chip *jz4730_msc_chip(l4_addr_t, l4_addr_t, l4_addr_t, Cpm_chip *);
//extern Msc_chip *jz4740_msc_chip(l4_addr_t, l4_addr_t, l4_addr_t, Cpm_chip *);
extern Msc_chip *jz4780_msc_chip(l4_addr_t, l4_addr_t, l4_addr_t, Cpm_chip *);
extern Msc_chip *x1600_msc_chip(l4_addr_t, l4_addr_t, l4_addr_t, Cpm_chip *);

struct msc_function
{
  const char *name;
  Msc_chip *(*function)(l4_addr_t, l4_addr_t, l4_addr_t, Cpm_chip *);
};

static struct msc_function functions[] = {
  //{"jz4730", jz4730_msc_chip},
  //{"jz4740", jz4740_msc_chip},
  {"jz4780", jz4780_msc_chip},
  {"x1600", x1600_msc_chip},
  {NULL, NULL}
};

Msc_chip *new_msc_chip(const char *name, l4_addr_t msc_start, l4_addr_t start,
                       l4_addr_t end, Cpm_chip *cpm)
{
  for (struct msc_function *f = functions; f->name != NULL; f++)
  {
    if (!strcmp(name, f->name))
      return f->function(msc_start, start, end, cpm);
  }

  return NULL;
}
